﻿using System.Collections.Generic;
using System.Linq;

namespace SampleCode
{
	public class LINQ
	{
		public void Foo()
		{
			// sample data
			var customers = AllCustomers();

			// .NET 2.0: query with custom iterator
			var bigCustomers = CustomersWithBalanceOver(customers, 1000);

			// .NET 3.5: use lambdas and LINQ (extension methods from Microsoft)
			var bigCustomers2 = customers.Where(c => c.Balance > 1000);

			// LINQ "Where" is a non-specialized version of our "CustomersWithBalanceOver"
			// iterator.  Can be used for many more things
			var ooCustomers = customers.Where(c => c.Name.Contains("oo"));

			// use "Select" extension method to return only certain parts
			// of each matching object
			var ooCustomerNames = customers.Where(c => c.Name.Contains("oo")).Select(c => c.Name);

			// LINQ has aggregate functions too.
			var totalOutstanding = customers.Sum(c => c.Balance);
			var highestBalance = customers.Max(c => c.Balance);
			var lowestBalance = customers.Min(c => c.Balance);

			// all samples above used "method syntax".  C# also has a "query syntax"
			// which is more SQL-like
			var bigCustomers3 = from c in customers
			                    where c.Balance > 1000
			                    select c;

		}

		#region Iterator
		public IEnumerable<Customer> CustomersWithBalanceOver(IEnumerable<Customer> customers,
			decimal balance)
		{
			foreach (var c in customers)
			{
				if (c.Balance > balance)
				{
					yield return c;
				}
			}
		}

		#endregion

		#region Sample Data
		private IEnumerable<Customer> AllCustomers()
		{
			return new[]
			       {
			       	new Customer {ID = 1, Name = "Microsoft", Balance = 50},
			       	new Customer {ID = 2, Name = "Apple", Balance = 0},
			       	new Customer {ID = 3, Name = "Yahoo", Balance = 300},
			       	new Customer {ID = 3, Name = "Google", Balance = 5000},
			       };
		}
		#endregion
	}

	public class Customer
	{
		public int ID { get; set; }
		public string Name { get; set; }
		public decimal Balance { get; set; }
	}
}
